@@ -1,5 +1,4 @@
 from collections import OrderedDict
-from itertools import compress
 
 try:
     import itertools.izip as zip
@@ -8,9 +7,9 @@ except ImportError:
 
 import numpy as np
 
-from .interface import Interface
+from .interface import Interface, DataError
 from ..dimension import Dimension
-from ..element import Element, NdElement
+from ..element import Element
 from ..dimension import OrderedDict as cyODict
 from ..ndmapping import NdMapping, item_check
 from .. import util
@@ -30,7 +29,9 @@ class DictInterface(Interface):
     @classmethod
     def dimension_type(cls, dataset, dim):
         name = dataset.get_dimension(dim, strict=True).name
-        return dataset.data[name].dtype.type
+        values = dataset.data[name]
+        return type(values) if np.isscalar(values) else values.dtype.type
+
 
     @classmethod
     def init(cls, eltype, data, kdims, vdims):
@@ -44,12 +45,11 @@ class DictInterface(Interface):
                       d for d in kdims + vdims]
         if isinstance(data, tuple):
             data = {d: v for d, v in zip(dimensions, data)}
-        elif ((util.is_dataframe(data) and all(d in data for d in dimensions)) or
-              (isinstance(data, NdElement) and all(d in data.dimensions() for d in dimensions))):
+        elif util.is_dataframe(data) and all(d in data for d in dimensions):
             data = {d: data[d] for d in dimensions}
         elif isinstance(data, np.ndarray):
             if data.ndim == 1:
-                if eltype._auto_indexable_1d:
+                if eltype._auto_indexable_1d and len(kdims)+len(vdims)>1:
                     data = np.column_stack([np.arange(len(data)), data])
                 else:
                     data = np.atleast_2d(data).T
@@ -68,33 +68,57 @@ class DictInterface(Interface):
         elif not any(isinstance(data, tuple(t for t in interface.types if t is not None))
                      for interface in cls.interfaces.values()):
             data = {k: v for k, v in zip(dimensions, zip(*data))}
-        elif isinstance(data, dict) and not all(d in data for d in dimensions):
+        elif (isinstance(data, dict) and not any(isinstance(v, np.ndarray) for v in data.values()) and not
+              any(d in data or any(d in k for k in data if isinstance(k, tuple)) for d in dimensions)):
+            # For data where both keys and values are dimension values
+            # e.g. {('A', 'B'): (1, 2)} (should consider deprecating)
+            dict_data = sorted(data.items())
+            k, v = dict_data[0]
+            if len(util.wrap_tuple(k)) != len(kdims) or len(util.wrap_tuple(v)) != len(vdims):
+                raise ValueError("Dictionary data not understood, should contain a column "
+                                 "per dimension or a mapping between key and value dimension "
+                                 "values.")
             dict_data = zip(*((util.wrap_tuple(k)+util.wrap_tuple(v))
-                              for k, v in data.items()))
+                              for k, v in dict_data))
             data = {k: np.array(v) for k, v in zip(dimensions, dict_data)}
 
         if not isinstance(data, cls.types):
             raise ValueError("DictInterface interface couldn't convert data.""")
         elif isinstance(data, dict):
-            unpacked = [(d, np.array(data[d])) for d in data]
-            if not cls.expanded([d[1] for d in unpacked]):
+            unpacked = []
+            for d, vals in data.items():
+                if isinstance(d, tuple):
+                    vals = np.asarray(vals)
+                    if not vals.ndim == 2 and vals.shape[1] == len(d):
+                        raise ValueError("Values for %s dimensions did not have "
+                                         "the expected shape.")
+                    for i, sd in enumerate(d):
+                        unpacked.append((sd, vals[:, i]))
+                else:
+                    unpacked.append((d, vals if np.isscalar(vals) else np.asarray(vals)))
+            if not cls.expanded([d[1] for d in unpacked if not np.isscalar(d[1])]):
                 raise ValueError('DictInterface expects data to be of uniform shape.')
             if isinstance(data, odict_types):
                 data.update(unpacked)
             else:
-                data = OrderedDict([(d, np.array(data[d])) for d in dimensions])
+                data = OrderedDict(unpacked)
         return data, {'kdims':kdims, 'vdims':vdims}, {}
 
 
     @classmethod
-    def validate(cls, dataset):
-        dimensions = dataset.dimensions(label='name')
+    def validate(cls, dataset, vdims=True):
+        dim_types = 'key' if vdims else 'all'
+        dimensions = dataset.dimensions(dim_types, label='name')
         not_found = [d for d in dimensions if d not in dataset.data]
         if not_found:
-            raise ValueError('Following dimensions not found in data: %s' % not_found)
-        lengths = [len(dataset.data[dim]) for dim in dimensions]
-        if len({l for l in lengths if l > 1}) > 1:
-            raise ValueError('Length of columns do not match')
+            raise DataError('Following columns specified as dimensions '
+                            'but not found in data: %s' % not_found, cls)
+        lengths = [(dim, 1 if np.isscalar(dataset.data[dim]) else len(dataset.data[dim]))
+                   for dim in dimensions]
+        if len({l for d, l in lengths if l > 1}) > 1:
+            lengths = ', '.join(['%s: %d' % l for l in sorted(lengths)])
+            raise DataError('Length of columns must be equal or scalar, '
+                            'columns have lengths: %s' % lengths, cls)
 
 
     @classmethod
@@ -110,13 +134,20 @@ class DictInterface(Interface):
         if len(data[key]) == 1 and key in dataset.vdims:
             return data[key][0]
 
+    @classmethod
+    def isscalar(cls, dataset, dim):
+        name = dataset.get_dimension(dim, strict=True).name
+        values = dataset.data[name]
+        return np.isscalar(values) or len(np.unique(values)) == 1
+
     @classmethod
     def shape(cls, dataset):
         return cls.length(dataset), len(dataset.data),
 
     @classmethod
     def length(cls, dataset):
-        return len(list(dataset.data.values())[0])
+        lengths = [len(vals) for vals in dataset.data.values() if not np.isscalar(vals)]
+        return max(lengths) if lengths else 1
 
     @classmethod
     def array(cls, dataset, dimensions):
@@ -124,21 +155,28 @@ class DictInterface(Interface):
             dimensions = dataset.dimensions(label='name')
         else:
             dimensions = [dataset.get_dimensions(d).name for d in dimensions]
-        return np.column_stack(dataset.data[dim.name] for dim in dimensions)
+        arrays = [dataset.data[dim.name] for dim in dimensions]
+        return np.column_stack([np.full(len(dataset), arr) if np.isscalar(arr) else arr
+                                for arr in arrays])
 
     @classmethod
     def add_dimension(cls, dataset, dimension, dim_pos, values, vdim):
         dim = dimension.name if isinstance(dimension, Dimension) else dimension
         data = list(dataset.data.items())
-        if isinstance(values, util.basestring) or not hasattr(values, '__iter__'):
-            values = np.array([values]*len(dataset))
         data.insert(dim_pos, (dim, values))
         return OrderedDict(data)
 
     @classmethod
     def redim(cls, dataset, dimensions):
-        return OrderedDict([(dimensions.get(k, dataset.get_dimension(k)).name, v)
-                            for k,v in dataset.data.items()])
+        all_dims = dataset.dimensions()
+        renamed = []
+        for k, v in dataset.data.items():
+            if k in dimensions:
+                k = dimensions[k].name
+            elif k in all_dims:
+                k = dataset.get_dimension(k).name
+            renamed.append((k, v))
+        return OrderedDict(renamed)
 
     @classmethod
     def concat(cls, dataset_objs):
@@ -154,22 +192,29 @@ class DictInterface(Interface):
 
 
     @classmethod
-    def sort(cls, dataset, by=[]):
+    def sort(cls, dataset, by=[], reverse=False):
         by = [dataset.get_dimension(d).name for d in by]
         if len(by) == 1:
             sorting = cls.values(dataset, by[0]).argsort()
         else:
             arrays = [dataset.dimension_values(d) for d in by]
             sorting = util.arglexsort(arrays)
-        return OrderedDict([(d, v[sorting]) for d, v in dataset.data.items()])
+        return OrderedDict([(d, v if np.isscalar(v) else (v[sorting][::-1] if reverse else v[sorting]))
+                            for d, v in dataset.data.items()])
 
 
     @classmethod
     def values(cls, dataset, dim, expanded=True, flat=True):
         dim = dataset.get_dimension(dim).name
-        values = np.array(dataset.data.get(dim))
-        if not expanded:
-            return util.unique_array(values)
+        values = dataset.data.get(dim)
+        if np.isscalar(values):
+            if not expanded:
+                return np.array([values])
+            values = np.full(len(dataset), values, dtype=np.array(values).dtype)
+        else:
+            if not expanded:
+                return util.unique_array(values)
+            values = np.array(values)
         return values
 
 
@@ -196,14 +241,17 @@ class DictInterface(Interface):
         group_kwargs.update(kwargs)
 
         # Find all the keys along supplied dimensions
-        keys = [tuple(dataset.data[d.name][i] for d in dimensions)
-                for i in range(len(dataset))]
+        keys = (tuple(dataset.data[d.name] if np.isscalar(dataset.data[d.name])
+                      else dataset.data[d.name][i] for d in dimensions)
+                for i in range(len(dataset)))
 
         # Iterate over the unique entries applying selection masks
         grouped_data = []
         for unique_key in util.unique_iterator(keys):
             mask = cls.select_mask(dataset, dict(zip(dimensions, unique_key)))
-            group_data = OrderedDict(((d.name, dataset[d.name][mask]) for d in kdims+vdims))
+            group_data = OrderedDict(((d.name, dataset.data[d.name] if np.isscalar(dataset.data[d.name])
+                                       else dataset.data[d.name][mask])
+                                      for d in kdims+vdims))
             group_data = group_type(group_data, **group_kwargs)
             grouped_data.append((unique_key, group_data))
 
@@ -219,10 +267,11 @@ class DictInterface(Interface):
         if selection_mask is None:
             selection_mask = cls.select_mask(dataset, selection)
         indexed = cls.indexed(dataset, selection)
-        data = OrderedDict((k, list(compress(v, selection_mask)))
+        data = OrderedDict((k, v if np.isscalar(v) else v[selection_mask])
                            for k, v in dataset.data.items())
         if indexed and len(list(data.values())[0]) == 1 and len(dataset.vdims) == 1:
-            return data[dataset.vdims[0].name][0]
+            value = data[dataset.vdims[0].name]
+            return value if np.isscalar(value) else value[0]
         return data
 
 
@@ -234,9 +283,9 @@ class DictInterface(Interface):
             if np.isscalar(sample): sample = [sample]
             for i, v in enumerate(sample):
                 name = dataset.get_dimension(i).name
-                sample_mask &= (np.array(dataset.data[name])==v)
+                sample_mask &= (dataset.data[name]==v)
             mask |= sample_mask
-        return {k: np.array(col)[mask]
+        return {k: col if np.isscalar(col) else np.array(col)[mask]
                 for k, col in dataset.data.items()}
 
 
@@ -253,7 +302,9 @@ class DictInterface(Interface):
                 aggregated[kdim].append(val)
             for vdim, arr in group.items():
                 if vdim in dataset.vdims:
-                    if isinstance(function, np.ufunc):
+                    if np.isscalar(arr):
+                        reduced = arr
+                    elif isinstance(function, np.ufunc):
                         reduced = function.reduce(arr, **kwargs)
                     else:
                         reduced = function(arr, **kwargs)
@@ -261,4 +312,33 @@ class DictInterface(Interface):
         return aggregated
 
 
+    @classmethod
+    def iloc(cls, dataset, index):
+        rows, cols = index
+        scalar = False
+        if np.isscalar(cols):
+            scalar = np.isscalar(rows)
+            cols = [dataset.get_dimension(cols, strict=True)]
+        elif isinstance(cols, slice):
+            cols = dataset.dimensions()[cols]
+        else:
+            cols = [dataset.get_dimension(d, strict=True) for d in cols]
+
+        if np.isscalar(rows):
+            rows = [rows]
+
+        new_data = OrderedDict()
+        for d, values in dataset.data.items():
+            if d in cols:
+                if np.isscalar(values):
+                    new_data[d] = values
+                else:
+                    new_data[d] = values[rows]
+
+        if scalar:
+            arr = new_data[cols[0].name]
+            return arr if np.isscalar(arr) else arr[0]
+        return new_data
+
+
 Interface.register(DictInterface)
