@@ -10,6 +10,7 @@ import datetime as dt
 from collections import defaultdict
 from functools import partial
 from contextlib import contextmanager
+from distutils.version import LooseVersion
 
 from threading import Thread, Event
 import numpy as np
@@ -27,11 +28,17 @@ try:
 except:
    import builtins as builtins   # noqa (compatibility)
 
-datetime_types = (np.datetime64, dt.datetime)
+datetime_types = (np.datetime64, dt.datetime, dt.date)
+timedelta_types = (np.timedelta64, dt.timedelta,)
 
 try:
-    import pandas as pd # noqa (optional import)
-    datetime_types = datetime_types + (pd.Timestamp,)
+    import pandas as pd
+    if LooseVersion(pd.__version__) > '0.20.0':
+        from pandas.core.dtypes.dtypes import DatetimeTZDtypeType
+    else:
+        from pandas.types.dtypes import DatetimeTZDtypeType
+    datetime_types = datetime_types + (pd.Timestamp, DatetimeTZDtypeType)
+    timedelta_types = timedelta_types + (pd.Timedelta,)
 except ImportError:
     pd = None
 
@@ -41,6 +48,13 @@ except ImportError:
     dd = None
 
 
+class VersionError(Exception):
+    "Raised when there is a library version mismatch."
+    def __init__(self, msg, version=None, min_version=None, **kwargs):
+        self.version = version
+        self.min_version = min_version
+        super(VersionError, self).__init__(msg, **kwargs)
+
 
 class Config(param.ParameterizedFunction):
     """
@@ -96,7 +110,7 @@ class HashableJSON(json.JSONEncoder):
         elif isinstance(obj, np.ndarray):
             return obj.tolist()
         if pd and isinstance(obj, (pd.Series, pd.DataFrame)):
-            return repr(sorted(list(obj.to_dict().items())))
+            return obj.to_csv().encode('utf-8')
         elif isinstance(obj, self.string_hashable):
             return str(obj)
         elif isinstance(obj, self.repr_hashable):
@@ -193,6 +207,7 @@ def deephash(obj):
 if sys.version_info.major == 3:
     basestring = str
     unicode = str
+    long = int
     generator_types = (zip, range, types.GeneratorType)
 else:
     basestring = basestring
@@ -710,6 +725,18 @@ def max_range(ranges):
         return (np.NaN, np.NaN)
 
 
+def dimension_range(lower, upper, dimension):
+    """
+    Computes the range along a dimension by combining the data range
+    with the Dimension soft_range and range.
+    """
+    lower, upper = max_range([(lower, upper), dimension.soft_range])
+    dmin, dmax = dimension.range
+    lower = lower if dmin is None or not np.isfinite(dmin) else dmin
+    upper = upper if dmax is None or not np.isfinite(dmax) else dmax
+    return lower, upper
+
+
 def max_extents(extents, zrange=False):
     """
     Computes the maximal extent in 2D and 3D space from
@@ -874,7 +901,7 @@ def merge_dimensions(dimensions_list):
     return [d(values=dvalues.get(d.name, [])) for d in dimensions]
 
 
-def dimension_sort(odict, kdims, vdims, categorical, key_index, cached_values):
+def dimension_sort(odict, kdims, vdims, key_index):
     """
     Sorts data by key using usual Python tuple sorting semantics
     or sorts in categorical order for any categorical Dimensions.
@@ -885,16 +912,14 @@ def dimension_sort(odict, kdims, vdims, categorical, key_index, cached_values):
     indexes = [(dimensions[i], int(i not in range(ndims)),
                     i if i in range(ndims) else i-ndims)
                 for i in key_index]
-    cached_values = {d: [None]+vals for d, vals in cached_values.items()}
+    cached_values = {d.name: [None]+list(d.values) for d in dimensions}
 
     if len(set(key_index)) != len(key_index):
         raise ValueError("Cannot sort on duplicated dimensions")
-    elif categorical:
+    else:
        sortkws['key'] = lambda x: tuple(cached_values[dim.name].index(x[t][d])
                                         if dim.values else x[t][d]
                                         for i, (dim, t, d) in enumerate(indexes))
-    elif key_index != list(range(len(kdims+vdims))):
-        sortkws['key'] = lambda x: tuple(x[t][d] for _, t, d in indexes)
     if sys.version_info.major == 3:
         return python2sort(odict.items(), **sortkws)
     else:
@@ -1471,7 +1496,7 @@ def is_nan(x):
         return False
 
 
-def bound_range(vals, density):
+def bound_range(vals, density, time_unit='us'):
     """
     Computes a bounding range and density from a number of samples
     assumed to be evenly spaced. Density is rounded to machine precision
@@ -1484,11 +1509,76 @@ def bound_range(vals, density):
     if not density:
         with warnings.catch_warnings():
             warnings.filterwarnings('ignore', r'invalid value encountered in double_scalars')
-            full_precision_density = 1./((high-low)/(len(vals)-1))
+            full_precision_density = compute_density(low, high, len(vals)-1)
             density = round(full_precision_density, sys.float_info.dig)
         if density == 0:
             density = full_precision_density
     if density == 0:
         raise ValueError('Could not determine Image density, ensure it has a non-zero range.')
     halfd = 0.5/density
+    if isinstance(low, datetime_types):
+        halfd = np.timedelta64(int(round(halfd)), time_unit)
     return low-halfd, high+halfd, density, invert
+
+
+def compute_density(start, end, length, time_unit='us'):
+    """
+    Computes a grid density given the edges and number of samples.
+    Handles datetime grids correctly by computing timedeltas and
+    computing a density for the given time_unit.
+    """
+    if isinstance(start, int): start = float(start)
+    if isinstance(end, int): end = float(end)
+    diff = end-start
+    if isinstance(diff, timedelta_types):
+        if isinstance(diff, np.timedelta64):
+            diff = np.timedelta64(diff, time_unit).tolist()
+        tscale = 1./np.timedelta64(1, time_unit).tolist().total_seconds()
+        return (length/(diff.total_seconds()*tscale))
+    else:
+        return length/diff
+
+
+def date_range(start, end, length, time_unit='us'):
+    """
+    Computes a date range given a start date, end date and the number
+    of samples.
+    """
+    step = (1./compute_density(start, end, length, time_unit))
+    if pd and isinstance(start, pd.Timestamp):
+        start = start.to_datetime64()
+    step = np.timedelta64(int(round(step)), time_unit)
+    return start+step/2.+np.arange(length)*step
+
+
+def dt_to_int(value, time_unit='us'):
+    """
+    Converts a datetime type to an integer with the supplied time unit.
+    """
+    if time_unit == 'ns':
+        tscale = 1./np.timedelta64(1, time_unit).tolist()
+    else:
+        tscale = 1./np.timedelta64(1, time_unit).tolist().total_seconds()
+    if pd and isinstance(value, pd.Timestamp):
+        value = value.to_pydatetime()
+    elif isinstance(value, np.datetime64):
+        value = value.tolist()
+    if isinstance(value, (int, long)):
+        # Handle special case of nanosecond precision which cannot be
+        # represented by python datetime
+        return value * 10**-(np.log10(tscale)-3)
+    try:
+        # Handle python3
+        return int(value.timestamp() * tscale)
+    except:
+        # Handle python2
+        return (time.mktime(value.timetuple()) + value.microsecond / 1e6) * tscale
+
+
+def search_indices(values, source):
+    """
+    Given a set of values returns the indices of each of those values
+    in the source array.
+    """
+    orig_indices = source.argsort()
+    return orig_indices[np.searchsorted(source[orig_indices], values)]
