@@ -118,7 +118,7 @@ class ResamplingOperation(Operation):
 
         xtype = 'numeric'
         if isinstance(xstart, datetime_types) or isinstance(xend, datetime_types):
-            xstart, xend = dt_to_int(xstart), dt_to_int(xend)
+            xstart, xend = dt_to_int(xstart, 'ns'), dt_to_int(xend, 'ns')
             xtype = 'datetime'
         elif not np.isfinite(xstart) and not np.isfinite(xend):
             if element.get_dimension_type(x) in datetime_types:
@@ -132,7 +132,7 @@ class ResamplingOperation(Operation):
 
         ytype = 'numeric'
         if isinstance(ystart, datetime_types) or isinstance(yend, datetime_types):
-            ystart, yend = dt_to_int(ystart), dt_to_int(yend)
+            ystart, yend = dt_to_int(ystart, 'ns'), dt_to_int(yend, 'ns')
             ytype = 'datetime'
         elif not np.isfinite(ystart) and not np.isfinite(yend):
             if element.get_dimension_type(y) in datetime_types:
@@ -247,8 +247,7 @@ class aggregate(ResamplingOperation):
             df = df.copy()
         for d in (x, y):
             if df[d.name].dtype.kind == 'M':
-                df[d.name] = df[d.name].astype('datetime64[ns]').astype('int64') * 10e-4
-
+                df[d.name] = df[d.name].astype('datetime64[ns]').astype('int64') * 1000.
         return x, y, Dataset(df, kdims=kdims, vdims=vdims), glyph
 
 
@@ -343,9 +342,15 @@ class aggregate(ResamplingOperation):
         cvs = ds.Canvas(plot_width=width, plot_height=height,
                         x_range=x_range, y_range=y_range)
 
-        column = agg_fn.column
-        if column and isinstance(agg_fn, ds.count_cat):
-            name = '%s Count' % agg_fn.column
+        column = agg_fn.column if agg_fn else None
+        if column:
+            dims = [d for d in element.dimensions('ranges') if d == column]
+            if not dims:
+                raise ValueError("Aggregation column %s not found on %s element. "
+                                 "Ensure the aggregator references an existing "
+                                 "dimension." % (column,element))
+            name = '%s Count' % column if isinstance(agg_fn, ds.count_cat) else column
+            vdims = [dims[0](name)]
         else:
             name = column
         vdims = [element.get_dimension(column)(name) if column
@@ -355,12 +360,12 @@ class aggregate(ResamplingOperation):
 
         dfdata = PandasInterface.as_dframe(data)
         agg = getattr(cvs, glyph)(dfdata, x.name, y.name, self.p.aggregator)
-        if 'x_axis' in agg and 'y_axis' in agg:
+        if 'x_axis' in agg.coords and 'y_axis' in agg.coords:
             agg = agg.rename({'x_axis': x, 'y_axis': y})
         if xtype == 'datetime':
-            agg[x.name] = agg[x.name].astype('datetime64[us]')
+            agg[x.name] = (agg[x.name]/10e5).astype('datetime64[us]')
         if ytype == 'datetime':
-            agg[y.name] = agg[y.name].astype('datetime64[us]')
+            agg[y.name] = (agg[y.name]/10e5).astype('datetime64[us]')
 
         if agg.ndim == 2:
             # Replacing x and y coordinates to avoid numerical precision issues
@@ -371,7 +376,7 @@ class aggregate(ResamplingOperation):
             for c in agg.coords[column].data:
                 cagg = agg.sel(**{column: c})
                 eldata = cagg if ds_version > '0.5.0' else (xs, ys, cagg.data)
-                layers[c] = self.p.element_type(eldata, **params)
+                layers[c] = self.p.element_type(eldata, **dict(params, vdims=vdims))
             return NdOverlay(layers, kdims=[data.get_dimension(column)])
 
 
@@ -415,33 +420,41 @@ class regrid(ResamplingOperation):
         the width and height are clipped to what is available on the
         source array.""")
 
-    def _process(self, element, key=None):
-        if ds_version <= '0.5.0':
-            raise RuntimeError('regrid operation requires datashader>=0.6.0')
 
+    def _get_xarrays(self, element, coords, xtype, ytype):
         x, y = element.kdims
-        (x_range, y_range), _, (width, height), (xtype, ytype) = self._get_sampling(element, x, y)
-
-        coords = tuple(element.dimension_values(d, expanded=False)
-                       for d in [x, y])
-        coord_dict = {x.name: coords[0], y.name: coords[1]}
         dims = [y.name, x.name]
-        arrays = []
+        coord_dict = {x.name: coords[0], y.name: coords[1]}
+
+        arrays = {}
         for vd in element.vdims:
             if element.interface is XArrayInterface:
                 xarr = element.data[vd.name]
                 if 'datetime' in (xtype, ytype):
                     xarr = xarr.copy()
-                if dims != xarr.dims:
+                if dims != xarr.dims and not irregular:
                     xarr = xarr.transpose(*dims)
             else:
                 arr = element.dimension_values(vd, flat=False)
                 xarr = xr.DataArray(arr, coords=coord_dict, dims=dims)
             if xtype == "datetime":
-                xarr[x.name] = [dt_to_int(v) for v in xarr[x.name].values]
+                xarr[x.name] = [dt_to_int(v, 'ns') for v in xarr[x.name].values]
             if ytype == "datetime":
-                xarr[y.name] = [dt_to_int(v) for v in xarr[y.name].values]
-            arrays.append(xarr)
+                xarr[y.name] = [dt_to_int(v, 'ns') for v in xarr[y.name].values]
+            arrays[vd.name] = xarr
+        return arrays
+
+
+    def _process(self, element, key=None):
+        if ds_version <= '0.5.0':
+            raise RuntimeError('regrid operation requires datashader>=0.6.0')
+
+        x, y = element.kdims
+        coords = tuple(element.dimension_values(d, expanded=False)
+                       for d in [x, y])
+        info = self._get_sampling(element, x, y)
+        (x_range, y_range), _, (width, height), (xtype, ytype) = info
+        arrays = self._get_xarrays(element, coords, xtype, ytype)
 
         # Disable upsampling if requested
         (xstart, xend), (ystart, yend) = (x_range, y_range)
@@ -449,9 +462,9 @@ class regrid(ResamplingOperation):
         if not self.p.upsample and self.p.target is None:
             (x0, x1), (y0, y1) = element.range(0), element.range(1)
             if isinstance(x0, datetime_types):
-                x0, x1 = dt_to_int(x0), dt_to_int(x1)
+                x0, x1 = dt_to_int(x0, 'ns'), dt_to_int(x1, 'ns')
             if isinstance(y0, datetime_types):
-                y0, y1 = dt_to_int(y0), dt_to_int(y1)
+                y0, y1 = dt_to_int(y0, 'ns'), dt_to_int(y1, 'ns')
             exspan, eyspan = (x1-x0), (y1-y0)
             width = min([int((xspan/exspan) * len(coords[0])), width])
             height = min([int((yspan/eyspan) * len(coords[1])), height])
@@ -459,24 +472,24 @@ class regrid(ResamplingOperation):
         # Get expanded or bounded ranges
         cvs = ds.Canvas(plot_width=width, plot_height=height,
                         x_range=x_range, y_range=y_range)
-        regridded = []
-        for xarr in arrays:
+        regridded = {}
+        for vd, xarr in arrays.items():
             rarray = cvs.raster(xarr, upsample_method=self.p.interpolation,
                                 downsample_method=self.p.aggregator)
             if xtype == "datetime":
-                rarray[x.name] = rarray[x.name].astype('datetime64[us]')
+                rarray[x.name] = (rarray[x.name]/10e5).astype('datetime64[us]')
             if ytype == "datetime":
-                rarray[y.name] = rarray[y.name].astype('datetime64[us]')
-            regridded.append(rarray)
+                rarray[y.name] = (rarray[y.name]/10e5).astype('datetime64[us]')
+            regridded[vd] = rarray
 
-        regridded = xr.Dataset({vd.name: xarr for vd, xarr in zip(element.vdims, regridded)})
+        regridded = xr.Dataset(regridded)
         if xtype == 'datetime':
-            xstart, xend = np.array([xstart, xend]).astype('datetime64[us]')
+            xstart, xend = (np.array([xstart, xend])/10e5).astype('datetime64[us]')
         if ytype == 'datetime':
-            ystart, yend = np.array([ystart, yend]).astype('datetime64[us]')  
+            ystart, yend = (np.array([ystart, yend])/10e5).astype('datetime64[us]')
         bbox = BoundingBox(points=[(xstart, ystart), (xend, yend)])
-        return element.clone(regridded, bounds=bbox, datatype=['xarray'])
-
+        return element.clone(regridded, bounds=bbox,
+                             datatype=['xarray']+element.datatype)
 
 
 class shade(Operation):
@@ -539,12 +552,12 @@ class shade(Operation):
         """
         if not isinstance(overlay, NdOverlay):
             raise ValueError('Only NdOverlays can be concatenated')
-        xarr = xr.concat([v.data.T for v in overlay.values()],
+        xarr = xr.concat([v.data.transpose() for v in overlay.values()],
                          pd.Index(overlay.keys(), name=overlay.kdims[0].name))
         params = dict(get_param_values(overlay.last),
                       vdims=overlay.last.vdims,
                       kdims=overlay.kdims+overlay.last.kdims)
-        return Dataset(xarr.T, datatype=['xarray'], **params)
+        return Dataset(xarr.transpose(), datatype=['xarray'], **params)
 
 
     @classmethod
@@ -565,7 +578,20 @@ class shade(Operation):
         return "#{0:02x}{1:02x}{2:02x}".format(*(int(v*255) for v in rgb))
 
 
+    @classmethod
+    def to_xarray(cls, element):
+        if issubclass(element.interface, XArrayInterface):
+            return element
+        data = tuple(element.dimension_values(kd, expanded=False)
+                     for kd in element.kdims)
+        data += tuple(element.dimension_values(vd, flat=False)
+                      for vd in element.vdims)
+        dtypes = [dt for dt in element.datatype if dt != 'xarray']
+        return element.clone(data, datatype=['xarray']+dtypes)
+
+
     def _process(self, element, key=None):
+        element = element.map(self.to_xarray, Image)
         if isinstance(element, NdOverlay):
             bounds = element.last.bounds
             element = self.concatenate(element)
