@@ -56,6 +56,10 @@ class MessageCallback(object):
             except AttributeError:
                 self.comm = Comm(plot)
         self.source = source
+        self.reset()
+
+
+    def reset(self):
         self.handle_ids = defaultdict(dict)
         self.callbacks = []
         self.plot_handles = {}
@@ -366,7 +370,7 @@ class ServerCallback(MessageCallback):
         value change at once rather than firing off multiple plot updates.
         """
         self._queue.append((attr, old, new))
-        if not self._active:
+        if not self._active and self.plot.document:
             self.plot.document.add_timeout_callback(self.process_on_change, 50)
             self._active = True
 
@@ -377,10 +381,11 @@ class ServerCallback(MessageCallback):
         value change at once rather than firing off multiple plot updates.
         """
         self._queue.append((event))
-        if not self._active:
+        if not self._active and self.plot.document:
             self.plot.document.add_timeout_callback(self.process_on_event, 50)
             self._active = True
 
+
     def process_on_event(self):
         """
         Trigger callback change event and triggering corresponding streams.
@@ -388,7 +393,6 @@ class ServerCallback(MessageCallback):
         if not self._queue:
             self._active = False
             return
-        self._queue = []
         # Get unique event types in the queue
         events = list(OrderedDict([(event.event_name, event)
                                    for event in self._queue]).values())
