@@ -3,8 +3,10 @@ from collections import defaultdict
 import numpy as np
 import param
 from bokeh.models import (CategoricalColorMapper, CustomJS, HoverTool,
-                          FactorRange, Whisker, Band, Range1d)
+                          FactorRange, Whisker, Band, Range1d, Circle,
+                          VBar, HBar)
 from bokeh.models.tools import BoxSelectTool
+from bokeh.transform import jitter
 
 from ...core import Dataset, OrderedDict
 from ...core.dimension import Dimension
@@ -34,6 +36,9 @@ class PointPlot(LegendPlot, ColorbarPlot):
       Determines whether the `scaling_factor` should be applied to
       the width or area of each point (default: "area").""")
 
+    jitter = param.Number(default=None, bounds=(0, None), doc="""
+      The amount of jitter to apply to offset the points along the x-axis.""")
+
     scaling_factor = param.Number(default=1, bounds=(0, None), doc="""
       Scaling factor which is applied to either the width or area
       of each point, depending on the value of `scaling_method`.""")
@@ -77,7 +82,7 @@ class PointPlot(LegendPlot, ColorbarPlot):
         mapping = dict(x=dims[xidx], y=dims[yidx])
         data = {}
 
-        if not self.static_source:
+        if not self.static_source or self.batched:
             xdim, ydim = dims[xidx], dims[yidx]
             data[xdim] = element.dimension_values(xidx)
             data[ydim] = element.dimension_values(yidx)
@@ -91,6 +96,11 @@ class PointPlot(LegendPlot, ColorbarPlot):
         data.update(sdata)
         mapping.update(smapping)
 
+        if self.jitter:
+            axrange = 'y_range' if self.invert_axes else 'x_range'
+            mapping['x'] = jitter(dims[xidx], self.jitter,
+                                  range=self.handles[axrange])
+
         self._get_hover_data(data, element)
         return data, mapping, style
 
@@ -161,17 +171,18 @@ class VectorFieldPlot(ColorbarPlot):
     def _get_lengths(self, element, ranges):
         mag_dim = element.get_dimension(self.size_index)
         (x0, x1), (y0, y1) = (element.range(i) for i in range(2))
-        base_dist = get_min_distance(element)
         if mag_dim:
             magnitudes = element.dimension_values(mag_dim)
             _, max_magnitude = ranges[mag_dim.name]
             if self.normalize_lengths and max_magnitude != 0:
                 magnitudes = magnitudes / max_magnitude
             if self.rescale_lengths:
+                base_dist = get_min_distance(element)
                 magnitudes *= base_dist
         else:
             magnitudes = np.ones(len(element))
             if self.rescale_lengths:
+                base_dist = get_min_distance(element)
                 magnitudes *= base_dist
 
         return magnitudes
@@ -259,7 +270,7 @@ class CurvePlot(ElementPlot):
         xidx, yidx = (1, 0) if self.invert_axes else (0, 1)
         x = element.get_dimension(xidx).name
         y = element.get_dimension(yidx).name
-        if self.static_source:
+        if self.static_source and not self.batched:
             return {}, dict(x=x, y=y), style
 
         if 'steps' in self.interpolation:
@@ -353,16 +364,24 @@ class SideHistogramPlot(ColorbarPlot, HistogramPlot):
         Whether to display the plot title.""")
 
     default_tools = param.List(default=['save', 'pan', 'wheel_zoom',
-                                        'box_zoom', 'reset', 'ybox_select'],
+                                        'box_zoom', 'reset'],
         doc="A list of plugin tools to use on the plot.")
 
     _callback = """
-    color_mapper.low = cb_data['geometry']['y0'];
-    color_mapper.high = cb_data['geometry']['y1'];
-    source.trigger('change')
-    main_source.trigger('change')
+    color_mapper.low = cb_data['geometry']['{axis}0'];
+    color_mapper.high = cb_data['geometry']['{axis}1'];
+    source.change.emit()
+    main_source.change.emit()
     """
 
+    def __init__(self, *args, **kwargs):
+        super(SideHistogramPlot, self).__init__(*args, **kwargs)
+        if self.invert_axes:
+            self.default_tools.append('ybox_select')
+        else:
+            self.default_tools.append('xbox_select')
+
+
     def get_data(self, element, ranges, style):
         if self.invert_axes:
             mapping = dict(top='right', bottom='left', left=0, right='top')
@@ -406,18 +425,20 @@ class SideHistogramPlot(ColorbarPlot, HistogramPlot):
         handles = {'color_mapper': self.handles['color_mapper'],
                    'source': self.handles['source'],
                    'main_source': main_source}
+        axis = 'y' if self.invert_axes else 'x'
+        callback = self._callback.format(axis=axis)
         if box_select.callback:
-            box_select.callback.code += self._callback
+            box_select.callback.code += callback
             box_select.callback.args.update(handles)
         else:
-            box_select.callback = CustomJS(args=handles, code=self._callback)
+            box_select.callback = CustomJS(args=handles, code=callback)
         return ret
 
 
 
 class ErrorPlot(ElementPlot):
 
-    style_opts = line_properties
+    style_opts = line_properties + ['lower_head', 'upper_head']
 
     _mapping = dict(base="base", upper="upper", lower="lower")
 
@@ -469,15 +490,35 @@ class ErrorPlot(ElementPlot):
 
 
 
-class SpreadPlot(ErrorPlot):
+class SpreadPlot(ElementPlot):
 
     style_opts = line_properties + fill_properties
-    _plot_methods = dict(single=Band)
+    _plot_methods = dict(single='patch')
+
+    def get_data(self, element, ranges, style):
+         mapping = dict(x='x', y='y')
+         xvals = element.dimension_values(0)
+         mean = element.dimension_values(1)
+         neg_error = element.dimension_values(2)
+         pos_idx = 3 if len(element.dimensions()) > 3 else 2
+         pos_error = element.dimension_values(pos_idx)
+
+         lower = mean - neg_error
+         upper = mean + pos_error
+         band_x = np.append(xvals, xvals[::-1])
+         band_y = np.append(lower, upper[::-1])
+         if self.invert_axes:
+             data = dict(x=band_y, y=band_x)
+         else:
+             data = dict(x=band_x, y=band_y)
+         return data, mapping, style
 
 
 
 class AreaPlot(SpreadPlot):
 
+    _stream_data = False # Plot does not support streaming data
+
     def get_extents(self, element, ranges):
         vdims = element.vdims
         vdim = vdims[0].name
@@ -488,23 +529,22 @@ class AreaPlot(SpreadPlot):
             ranges[vdim] = (np.nanmin([0, ranges[vdim][0]]), ranges[vdim][1])
         return super(AreaPlot, self).get_extents(element, ranges)
 
-    def get_data(self, element, ranges, style):
-        mapping = dict(self._mapping)
-        if self.static_source:
-            return {}, mapping, style
 
+    def get_data(self, element, ranges, style):
+        mapping = dict(x='x', y='y')
         xs = element.dimension_values(0)
+        x2 = np.hstack((xs[::-1], xs))
+
         if len(element.vdims) > 1:
-            lower = element.dimension_values(2)
+            bottom = element.dimension_values(2)
         else:
-            lower = np.zeros(len(element))
-        upper = element.dimension_values(1)
-        data = dict(base=xs, upper=upper, lower=lower)
+            bottom = np.zeros(len(element))
+        ys = np.hstack((bottom[::-1], element.dimension_values(1)))
 
         if self.invert_axes:
-            mapping['dimension'] = 'width'
+            data = dict(x=ys, y=x2)
         else:
-            mapping['dimension'] = 'height'
+            data = dict(x=x2, y=ys)
         return data, mapping, style
 
 
@@ -762,7 +802,7 @@ class BarPlot(ColorbarPlot, LegendPlot):
         # Merge data and mappings
         mapping.update(cmapping)
         for k, cd in cdata.items():
-            if self.color_index is None and cd.dtype.kind in 'if':
+            if isinstance(cmapper, CategoricalColorMapper) and cd.dtype.kind in 'if':
                 cd = categorize_array(cd, cdim)
             if k not in data or len(data[k]) != [len(data[key]) for key in data if key != k][0]:
                 data[k].append(cd)
@@ -979,6 +1019,11 @@ class BoxWhiskerPlot(CompositeElementPlot, ColorbarPlot, LegendPlot):
             xfactors, yfactors = factors, []
         return (yfactors, xfactors) if self.invert_axes else (xfactors, yfactors)
 
+    def _postprocess_hover(self, renderer, source):
+        if not isinstance(renderer.glyph, (Circle, VBar, HBar)):
+            return
+        super(BoxWhiskerPlot, self)._postprocess_hover(renderer, source)
+
     def get_data(self, element, ranges, style):
         if element.kdims:
             groups = element.groupby(element.kdims).data
@@ -987,7 +1032,8 @@ class BoxWhiskerPlot(CompositeElementPlot, ColorbarPlot, LegendPlot):
         vdim = dimension_sanitizer(element.vdims[0].name)
 
         # Define CDS data
-        r1_data, r2_data = ({'index': [], 'top': [], 'bottom': []} for i in range(2))
+        r1_data, r2_data = (defaultdict(list, {'index': [], 'top': [], 'bottom': []})
+                            for i in range(2))
         s1_data, s2_data = ({'x0': [], 'y0': [], 'x1': [], 'y1': []} for i in range(2))
         w1_data, w2_data = ({'index': [], vdim: []} for i in range(2))
         out_data = defaultdict(list, {'index': [], vdim: []})
@@ -1012,6 +1058,7 @@ class BoxWhiskerPlot(CompositeElementPlot, ColorbarPlot, LegendPlot):
             cidx = element.get_dimension_index(self.color_index)
         else:
             cdim, cidx = None, None
+        hover = any(isinstance(t, HoverTool) for t in self.state.tools)
 
         factors = []
         for key, g in groups.items():
@@ -1059,21 +1106,27 @@ class BoxWhiskerPlot(CompositeElementPlot, ColorbarPlot, LegendPlot):
             if len(outliers):
                 out_data['index'] += [label]*len(outliers)
                 out_data[vdim] += list(outliers)
-                if any(isinstance(t, HoverTool) for t in self.state.tools):
+                if hover:
                     for kd, k in zip(element.kdims, wrap_tuple(key)):
                         out_data[dimension_sanitizer(kd.name)] += [k]*len(outliers)
+            if hover:
+                for kd, k in zip(element.kdims, wrap_tuple(key)):
+                    r1_data[dimension_sanitizer(kd.name)].append(k)
+                    r2_data[dimension_sanitizer(kd.name)].append(k)
+                r1_data[vdim].append(q2)
+                r2_data[vdim].append(q2)
 
         # Define combined data and mappings
         bar_glyph = 'hbar' if self.invert_axes else 'vbar'
         data = {
             bar_glyph+'_1': r1_data, bar_glyph+'_2': r2_data, 'segment_1': s1_data,
             'segment_2': s2_data, 'rect_1': w1_data, 'rect_2': w2_data,
-            'circle': out_data
+            'circle_1': out_data
         }
         mapping = {
             bar_glyph+'_1': vbar_map, bar_glyph+'_2': vbar2_map, 'segment_1': seg_map,
             'segment_2': seg_map, 'rect_1': whisk_map, 'rect_2': whisk_map,
-            'circle': out_map
+            'circle_1': out_map
         }
 
         # Cast data to arrays to take advantage of base64 encoding
