@@ -5,7 +5,6 @@ import param
 import numpy as np
 import bokeh
 import bokeh.plotting
-from bokeh import palettes
 from bokeh.core.properties import value
 from bokeh.models import (HoverTool, Renderer, Range1d, DataRange1d, Title,
                           FactorRange, FuncTickFormatter, Tool, Legend)
@@ -24,10 +23,10 @@ from ...core.options import abbreviated_exception, SkipRendering
 from ...core import util
 from ...streams import Stream, Buffer
 from ..plot import GenericElementPlot, GenericOverlayPlot
-from ..util import dynamic_update
+from ..util import dynamic_update, process_cmap
 from .plot import BokehPlot, TOOLS
-from .util import (mpl_to_bokeh, get_tab_title, mplcmap_to_palette,
-                   py2js_tickformatter, rgba_tuple, recursive_model_update)
+from .util import (mpl_to_bokeh, get_tab_title,  py2js_tickformatter,
+                   rgba_tuple, recursive_model_update, glyph_order)
 
 property_prefixes = ['selection', 'nonselection', 'muted', 'hover']
 
@@ -50,7 +49,7 @@ legend_dimensions = ['label_standoff', 'label_width', 'label_height', 'glyph_wid
 
 class ElementPlot(BokehPlot, GenericElementPlot):
 
-    bgcolor = param.Parameter(default='white', doc="""
+    bgcolor = param.Parameter(default=None, allow_None=True, doc="""
         Background color of the plot.""")
 
     border = param.Number(default=10, doc="""
@@ -180,39 +179,6 @@ class ElementPlot(BokehPlot, GenericElementPlot):
         self._shared = {'x': False, 'y': False}
 
 
-    def _construct_callbacks(self):
-        """
-        Initializes any callbacks for streams which have defined
-        the plotted object as a source.
-        """
-        if isinstance(self, OverlayPlot):
-            zorders = []
-        elif self.batched:
-            zorders = list(range(self.zorder, self.zorder+len(self.hmap.last)))
-        else:
-            zorders = [self.zorder]
-
-        if isinstance(self, OverlayPlot) and not self.batched:
-            sources = []
-        elif not self.static or isinstance(self.hmap, DynamicMap):
-            sources = [(i, o) for i, inputs in self.stream_sources.items()
-                       for o in inputs if i in zorders]
-        else:
-            sources = [(self.zorder, self.hmap.last)]
-        cb_classes = set()
-        for _, source in sources:
-            streams = Stream.registry.get(id(source), [])
-            registry = Stream._callbacks['bokeh']
-            cb_classes |= {(registry[type(stream)], stream) for stream in streams
-                           if type(stream) in registry and stream.linked}
-        cbs = []
-        sorted_cbs = sorted(cb_classes, key=lambda x: id(x[0]))
-        for cb, group in groupby(sorted_cbs, lambda x: x[0]):
-            cb_streams = [s for _, s in group]
-            cbs.append(cb(self, cb_streams, source))
-        return cbs
-
-
     def _hover_opts(self, element):
         if self.batched:
             dims = list(self.hmap.last.kdims)
@@ -326,22 +292,26 @@ class ElementPlot(BokehPlot, GenericElementPlot):
         if plots and self.shared_axes and not norm_opts.get('axiswise', False):
             plot_ranges = self._merge_ranges(plots, xlabel, ylabel)
 
-        if el.get_dimension_type(0) in util.datetime_types:
-            x_axis_type = 'datetime'
-        else:
-            x_axis_type = 'log' if self.logx else 'auto'
-
-        if len(dims) > 1 and el.get_dimension_type(1) in util.datetime_types:
-            y_axis_type = 'datetime'
-        else:
-            y_axis_type = 'log' if self.logy else 'auto'
-
         # Get the Element that determines the range and get_extents
         range_el = el if self.batched and not isinstance(self, OverlayPlot) else element
         l, b, r, t = self.get_extents(range_el, ranges)
         if self.invert_axes:
             l, b, r, t = b, l, t, r
 
+        xtype = el.get_dimension_type(0)
+        if ((xtype is np.object_ and type(l) in util.datetime_types) or
+            xtype in util.datetime_types):
+            x_axis_type = 'datetime'
+        else:
+            x_axis_type = 'log' if self.logx else 'auto'
+
+        y_axis_type = 'log' if self.logy else 'auto'
+        if len(dims) > 1:
+            ytype = el.get_dimension_type(1)
+            if ((ytype is np.object_ and type(b) in util.datetime_types)
+                or ytype in util.datetime_types):
+                y_axis_type = 'datetime'
+
         # Declare shared axes
         if 'x_range' in plot_ranges:
             self._shared['x'] = True
@@ -668,9 +638,30 @@ class ElementPlot(BokehPlot, GenericElementPlot):
                                     self.overlay_dims.items()])
             else:
                 legend = element.label
-            properties['legend'] = value(legend)
+            if legend:
+                properties['legend'] = value(legend)
         return properties
 
+
+    def _filter_properties(self, properties, glyph_type, allowed):
+        glyph_props = dict(properties)
+        for gtype in ((glyph_type, '') if glyph_type else ('',)):
+            for prop in ('color', 'alpha'):
+                glyph_prop = properties.get(gtype+prop)
+                if glyph_prop and ('line_'+prop not in glyph_props or gtype):
+                    glyph_props['line_'+prop] = glyph_prop
+                if glyph_prop and ('fill_'+prop not in glyph_props or gtype):
+                    glyph_props['fill_'+prop] = glyph_prop
+
+            props = {k[len(gtype):]: v for k, v in glyph_props.items()
+                     if k.startswith(gtype)}
+            if self.batched:
+                glyph_props = dict(props, **glyph_props)
+            else:
+                glyph_props.update(props)
+        return {k: v for k, v in glyph_props.items() if k in allowed}
+
+
     def _update_glyph(self, renderer, properties, mapping, glyph):
         allowed_properties = glyph.properties()
         properties = mpl_to_bokeh(properties)
@@ -680,24 +671,7 @@ class ElementPlot(BokehPlot, GenericElementPlot):
                 glyph = getattr(renderer, glyph_type+'glyph', None)
             if not glyph or (not renderer and glyph_type):
                 continue
-            glyph_props = dict(merged)
-
-            for gtype in ((glyph_type, '') if glyph_type else ('',)):
-                for prop in ('color', 'alpha'):
-                    glyph_prop = merged.get(gtype+prop)
-                    if glyph_prop and ('line_'+prop not in glyph_props or gtype):
-                        glyph_props['line_'+prop] = glyph_prop
-                    if glyph_prop and ('fill_'+prop not in glyph_props or gtype):
-                        glyph_props['fill_'+prop] = glyph_prop
-
-                props = {k[len(gtype):]: v for k, v in glyph_props.items()
-                         if k.startswith(gtype)}
-                if self.batched:
-                    glyph_props = dict(props, **glyph_props)
-                else:
-                    glyph_props.update(props)
-            filtered = {k: v for k, v in glyph_props.items()
-                        if k in allowed_properties}
+            filtered = self._filter_properties(merged, glyph_type, allowed_properties)
             glyph.update(**filtered)
 
 
@@ -921,18 +895,24 @@ class CompositeElementPlot(ElementPlot):
     drawing of multiple glyphs.
     """
 
-    # Mapping between glyph name and style groups
+    # Mapping between glyph names and style groups
     _style_groups = {}
 
-    def _init_glyphs(self, plot, element, ranges, source):
+    # Defines the order in which glyphs are drawn, defined by glyph name
+    _draw_order = []
+
+    def _init_glyphs(self, plot, element, ranges, source, data=None, mapping=None, style=None):
         # Get data and initialize data source
-        style = self.style[self.cyclic_index]
-        data, mapping, style = self.get_data(element, ranges, style)
+        if None in (data, mapping):
+            style = self.style[self.cyclic_index]
+            data, mapping, style = self.get_data(element, ranges, style)
+
+        keys = glyph_order(dict(data, **mapping), self._draw_order)
 
         source_cache = {}
         current_id = element._plot_id
         self.handles['previous_id'] = current_id
-        for key in dict(mapping, **data):
+        for key in keys:
             ds_data = data.get(key, {})
             if id(ds_data) in source_cache:
                 source = source_cache[id(ds_data)]
@@ -941,7 +921,7 @@ class CompositeElementPlot(ElementPlot):
                 source_cache[id(ds_data)] = source
             self.handles[key+'_source'] = source
             properties = self._glyph_properties(plot, element, source, ranges, style)
-            properties = self._process_properties(key, properties)
+            properties = self._process_properties(key, properties, mapping.get(key, {}))
             with abbreviated_exception():
                 renderer, glyph = self._init_glyph(plot, mapping.get(key, {}), properties, key)
             self.handles[key+'_glyph'] = glyph
@@ -955,13 +935,16 @@ class CompositeElementPlot(ElementPlot):
                 self._update_glyph(renderer, properties, mapping.get(key, {}), glyph)
 
 
-    def _process_properties(self, key, properties):
+    def _process_properties(self, key, properties, mapping):
         key = '_'.join(key.split('_')[:-1]) if '_' in key else key
         style_group = self._style_groups[key]
         group_props = {}
         for k, v in properties.items():
             if k in self.style_opts:
-                if k.split('_')[0] == style_group:
+                group = k.split('_')[0]
+                if group == style_group:
+                    if k in mapping:
+                        v = mapping[k]
                     k = '_'.join(k.split('_')[1:])
                 else:
                     continue
@@ -983,7 +966,8 @@ class CompositeElementPlot(ElementPlot):
         style = self.style[self.cyclic_index]
         data, mapping, style = self.get_data(element, ranges, style)
 
-        for key in sorted(dict(mapping, **data)):
+        keys = glyph_order(dict(data, **mapping), self._draw_order)
+        for key in keys:
             gdata = data.get(key)
             source = self.handles[key+'_source']
             glyph = self.handles.get(key+'_glyph')
@@ -992,7 +976,7 @@ class CompositeElementPlot(ElementPlot):
 
             if glyph:
                 properties = self._glyph_properties(plot, element, source, ranges, style)
-                properties = self._process_properties(key, properties)
+                properties = self._process_properties(key, properties, mapping[key])
                 renderer = self.handles.get(key+'_glyph_renderer')
                 with abbreviated_exception():
                     self._update_glyph(renderer, properties, mapping[key], glyph)
@@ -1003,7 +987,7 @@ class CompositeElementPlot(ElementPlot):
         Returns a Bokeh glyph object.
         """
         properties = mpl_to_bokeh(properties)
-        plot_method = key.split('_')[0]
+        plot_method = '_'.join(key.split('_')[:-1])
         renderer = getattr(plot, plot_method)(**dict(properties, **mapping))
         return renderer, renderer.glyph
 
@@ -1090,14 +1074,15 @@ class ColorbarPlot(ElementPlot):
         self.handles['colorbar'] = color_bar
 
 
-    def _get_colormapper(self, dim, element, ranges, style, factors=None, colors=None):
+    def _get_colormapper(self, dim, element, ranges, style, factors=None, colors=None,
+                         name='color_mapper'):
         # The initial colormapper instance is cached the first time
         # and then only updated
-        if dim is None:
+        if dim is None and colors is None:
             return None
         if self.adjoined:
             cmappers = self.adjoined.traverse(lambda x: (x.handles.get('color_dim'),
-                                                         x.handles.get('color_mapper')))
+                                                         x.handles.get(name)))
             cmappers = [cmap for cdim, cmap in cmappers if cdim == dim]
             if cmappers:
                 cmapper = cmappers[0]
@@ -1107,30 +1092,18 @@ class ColorbarPlot(ElementPlot):
                 return None
 
         ncolors = None if factors is None else len(factors)
-        low, high = ranges.get(dim.name, element.range(dim.name))
-        if colors:
-            palette = colors
+        if dim:
+            low, high = ranges.get(dim.name, element.range(dim.name))
         else:
-            cmap = style.pop('cmap', 'viridis')
-            if isinstance(cmap, list):
-                palette = cmap
-            else:
-                try:
-                    # Process as matplotlib colormap
-                    palette = mplcmap_to_palette(cmap, ncolors)
-                except ValueError:
-                    # Process as bokeh palette
-                    palette = getattr(palettes, cmap, None)
-                    if isinstance(palette, dict):
-                        if ncolors in palette:
-                            palette = palette[ncolors]
-                        else:
-                            palette = sorted(palette.items())[-1][1]
+            low, high = None, None
+
+        cmap = colors or style.pop('cmap', 'viridis')
+        palette = process_cmap(cmap, ncolors)
         nan_colors = {k: rgba_tuple(v) for k, v in self.clipping_colors.items()}
         colormapper, opts = self._get_cmapper_opts(low, high, factors, nan_colors)
 
-        if 'color_mapper' in self.handles and isinstance(self.handles['color_mapper'], colormapper):
-            cmapper = self.handles['color_mapper']
+        cmapper = self.handles.get(name)
+        if cmapper is not None:
             if cmapper.palette != palette:
                 cmapper.palette = palette
             opts = {k: opt for k, opt in opts.items()
@@ -1139,27 +1112,34 @@ class ColorbarPlot(ElementPlot):
                 cmapper.update(**opts)
         else:
             cmapper = colormapper(palette=palette, **opts)
-            self.handles['color_mapper'] = cmapper
+            self.handles[name] = cmapper
             self.handles['color_dim'] = dim
         return cmapper
 
 
-    def _get_color_data(self, element, ranges, style, name='color', factors=None, colors=None):
+    def _get_color_data(self, element, ranges, style, name='color', factors=None, colors=None,
+                        int_categories=False):
         data, mapping = {}, {}
         cdim = element.get_dimension(self.color_index)
         if not cdim:
             return data, mapping
 
         cdata = element.dimension_values(cdim)
-        if factors is None and (isinstance(cdata, list) or cdata.dtype.kind in 'OSU'):
-            factors = list(np.unique(cdata))
+        field = util.dimension_sanitizer(cdim.name)
+        dtypes = 'iOSU' if int_categories else 'OSU'
+        if factors is None and (isinstance(cdata, list) or cdata.dtype.kind in dtypes):
+            factors = list(util.unique_array(cdata))
+        if factors and int_categories and cdata.dtype.kind == 'i':
+            field += '_str'
+            cdata = [str(f) for f in cdata]
+            factors = [str(f) for f in factors]
+
         mapper = self._get_colormapper(cdim, element, ranges, style,
                                        factors, colors)
-        data[cdim.name] = cdata
+        data[field] = cdata
         if factors is not None:
-            mapping['legend'] = {'field': cdim.name}
-        mapping[name] = {'field': cdim.name,
-                         'transform': mapper}
+            mapping['legend'] = {'field': field}
+        mapping[name] = {'field': field, 'transform': mapper}
         return data, mapping
 
 
@@ -1485,9 +1465,14 @@ class OverlayPlot(GenericOverlayPlot, LegendPlot):
 
         if element and not self.overlaid and not self.tabs and not self.batched:
             self._update_ranges(element, ranges)
-            
+
+        # Determine which stream (if any) triggered the update
+        triggering = [stream for stream in self.streams if stream._triggering]
+
         for k, subplot in self.subplots.items():
             el = None
+
+
             # If in Dynamic mode propagate elements to subplots
             if isinstance(self.hmap, DynamicMap) and element:
                 # In batched mode NdOverlay is passed to subplot directly
@@ -1498,6 +1483,11 @@ class OverlayPlot(GenericOverlayPlot, LegendPlot):
                     idx = dynamic_update(self, subplot, k, element, items)
                     if idx is not None:
                         _, el = items.pop(idx)
+
+                # Skip updates to subplots when its streams is not one of
+                # the streams that initiated the update
+                if triggering and all(s not in triggering for s in subplot.streams):
+                    continue
             subplot.update_frame(key, ranges, element=el)
 
         if not self.batched and isinstance(self.hmap, DynamicMap) and items:
