@@ -8,11 +8,13 @@ from bokeh.models import (ColumnDataSource, Column, Row, Div)
 from bokeh.models.widgets import Panel, Tabs
 
 from ...core import (OrderedDict, Store, GridMatrix, AdjointLayout,
-                     NdLayout, Empty, GridSpace, HoloMap, Element)
+                     NdLayout, Empty, GridSpace, HoloMap, Element,
+                     DynamicMap)
 from ...core.util import basestring, wrap_tuple, unique_iterator
 from ...element import Histogram
+from ...streams import Stream
 from ..plot import (DimensionedPlot, GenericCompositePlot, GenericLayoutPlot,
-                    GenericElementPlot)
+                    GenericElementPlot, GenericOverlayPlot)
 from ..util import attach_streams
 from .util import (layout_padding, pad_plots, filter_toolboxes, make_axis,
                    update_shared_sources, empty_plot)
@@ -99,6 +101,40 @@ class BokehPlot(DimensionedPlot):
         raise NotImplementedError
 
 
+    def _construct_callbacks(self):
+        """
+        Initializes any callbacks for streams which have defined
+        the plotted object as a source.
+        """
+        if isinstance(self, GenericOverlayPlot):
+            zorders = []
+        elif self.batched:
+            zorders = list(range(self.zorder, self.zorder+len(self.hmap.last)))
+        else:
+            zorders = [self.zorder]
+
+        if isinstance(self, GenericOverlayPlot) and not self.batched:
+            sources = []
+        elif not self.static or isinstance(self.hmap, DynamicMap):
+            sources = [(i, o) for i, inputs in self.stream_sources.items()
+                       for o in inputs if i in zorders]
+        else:
+            sources = [(self.zorder, self.hmap.last)]
+
+        cb_classes = set()
+        for _, source in sources:
+            streams = Stream.registry.get(id(source), [])
+            registry = Stream._callbacks['bokeh']
+            cb_classes |= {(registry[type(stream)], stream) for stream in streams
+                           if type(stream) in registry and stream.linked}
+        cbs = []
+        sorted_cbs = sorted(cb_classes, key=lambda x: id(x[0]))
+        for cb, group in groupby(sorted_cbs, lambda x: x[0]):
+            cb_streams = [s for _, s in group]
+            cbs.append(cb(self, cb_streams, source))
+        return cbs
+
+
     def push(self):
         """
         Pushes updated plot data via the Comm.
@@ -204,6 +240,9 @@ class BokehPlot(DimensionedPlot):
                     else:
                         renderer.update(source=new_source)
                     plot.handles['source'] = new_source
+                    for callback in plot.callbacks:
+                        callback.reset()
+                        callback.initialize()
                 shared_sources.append(new_source)
                 source_cols[id(new_source)] = [c for c in new_source.data]
         self.handles['shared_sources'] = shared_sources
@@ -291,6 +330,9 @@ class GridPlot(CompositePlot, GenericCompositePlot):
 
           {'title': '15pt'}""")
 
+    merge_tools = param.Boolean(default=True, doc="""
+        Whether to merge all the tools into a single toolbar""")
+
     shared_xaxis = param.Boolean(default=False, doc="""
         If enabled the x-axes of the GridSpace will be drawn from the
         objects inside the Grid rather than the GridSpace dimensions.""")
@@ -299,6 +341,13 @@ class GridPlot(CompositePlot, GenericCompositePlot):
         If enabled the x-axes of the GridSpace will be drawn from the
         objects inside the Grid rather than the GridSpace dimensions.""")
 
+    toolbar = param.ObjectSelector(default='above',
+                                   objects=["above", "below",
+                                            "left", "right", None],
+                                   doc="""
+        The toolbar location, must be one of 'above', 'below',
+        'left', 'right', None.""")
+
     xaxis = param.ObjectSelector(default=True,
                                  objects=['bottom', 'top', None, True, False], doc="""
         Whether and where to display the xaxis, supported options are
@@ -382,10 +431,6 @@ class GridPlot(CompositePlot, GenericCompositePlot):
             if not self.shared_yaxis:
                 kwargs['yaxis'] = None
 
-            if isinstance(layout, GridMatrix):
-                if view.traverse(lambda x: x, [Histogram]):
-                    kwargs['shared_axes'] = False
-
             # Create subplot
             plotting_class = Store.registry[self.renderer.backend].get(vtype, None)
             if plotting_class is None:
@@ -420,7 +465,8 @@ class GridPlot(CompositePlot, GenericCompositePlot):
             else:
                 passed_plots.append(None)
 
-        plot = gridplot(plots[::-1])
+        plot = gridplot(plots[::-1], toolbar_position=self.toolbar,
+                        merge_tools=self.merge_tools)
         plot = self._make_axes(plot)
 
         title = self._get_title(self.keys[-1])
@@ -507,9 +553,19 @@ class LayoutPlot(CompositePlot, GenericLayoutPlot):
         share their Bokeh data source allowing for linked brushing
         and other linked behaviors.""")
 
+    merge_tools = param.Boolean(default=True, doc="""
+        Whether to merge all the tools into a single toolbar""")
+
     tabs = param.Boolean(default=False, doc="""
         Whether to display overlaid plots in separate panes""")
 
+    toolbar = param.ObjectSelector(default='above',
+                                   objects=["above", "below",
+                                            "left", "right", None],
+                                   doc="""
+        The toolbar location, must be one of 'above', 'below',
+        'left', 'right', None.""")
+
     def __init__(self, layout, keys=None, **params):
         super(LayoutPlot, self).__init__(layout, keys=keys, **params)
         self.layout, self.subplots, self.paths = self._init_layout(layout)
@@ -715,7 +771,9 @@ class LayoutPlot(CompositePlot, GenericLayoutPlot):
         else:
             plots = filter_toolboxes(plots)
             plots, width = pad_plots(plots)
-            layout_plot = gridplot(children=plots, width=width, **kwargs)
+            layout_plot = gridplot(children=plots, width=width,
+                                   toolbar_position=self.toolbar,
+                                   merge_tools=self.merge_tools, **kwargs)
 
         title = self._get_title(self.keys[-1])
         if title:
