@@ -1,10 +1,14 @@
-from bokeh.models.widgets import DataTable, TableColumn
-
 import param
 
-from ...core import Dataset
+from bokeh.models.widgets import (
+    DataTable, TableColumn, NumberEditor, NumberFormatter, DateFormatter,
+    DateEditor, StringFormatter, StringEditor, IntEditor
+)
+
+from ...core import Dataset, Dimension
 from ...element import ItemTable
 from ...streams import Buffer
+from ...core.util import dimension_sanitizer, datetime_types
 from ..plot import GenericElementPlot
 from .plot import BokehPlot
 
@@ -30,9 +34,9 @@ class TablePlot(BokehPlot, GenericElementPlot):
         self.handles = {} if plot is None else self.handles['plot']
         element_ids = self.hmap.traverse(lambda x: id(x), [Dataset, ItemTable])
         self.static = len(set(element_ids)) == 1 and len(self.keys) == len(self.hmap)
-        self.callbacks = [] # Callback support on tables not implemented
+        self.callbacks = self._construct_callbacks()
         self.streaming = [s for s in self.streams if isinstance(s, Buffer)]
-
+        self.static_source = False
 
     def _execute_hooks(self, element):
         """
@@ -46,12 +50,8 @@ class TablePlot(BokehPlot, GenericElementPlot):
 
 
     def get_data(self, element, ranges, style):
-        dims = element.dimensions()
-        mapping = {d.name: d.name for d in dims}
-        data = {d: element.dimension_values(d) for d in dims}
-        data = {d.name: values if values.dtype.kind in "if" else list(map(d.pprint_value, values))
-                for d, values in data.items()}
-        return data, mapping, style
+        return ({dimension_sanitizer(d.name): element.dimension_values(d)
+                 for d in element.dimensions()}, {}, style)
 
 
     def initialize_plot(self, ranges=None, plot=None, plots=None, source=None):
@@ -70,8 +70,28 @@ class TablePlot(BokehPlot, GenericElementPlot):
             source = self._init_datasource(data)
         self.handles['source'] = source
 
+        columns = []
         dims = element.dimensions()
-        columns = [TableColumn(field=d.name, title=d.pprint_label) for d in dims]
+        for d in dims:
+            col = dimension_sanitizer(d.name)
+            kind = data[col].dtype.kind
+            if kind == 'i':
+                formatter = NumberFormatter()
+                editor = IntEditor()
+            elif kind == 'f':
+                formatter = NumberFormatter(format='0,0.0[00000]')
+                editor = NumberEditor()
+            elif kind == 'M' or (kind == 'O' and len(data[col]) and type(data[col][0]) in datetime_types):
+                dimtype = element.get_dimension_type(0)
+                dformat = Dimension.type_formatters.get(dimtype, '%Y-%m-%d %H:%M:%S')
+                formatter = DateFormatter(format=dformat)
+                editor = DateEditor()
+            else:
+                formatter = StringFormatter()
+                editor = StringEditor()
+            column = TableColumn(field=d.name, title=d.pprint_label,
+                                 editor=editor, formatter=formatter)
+            columns.append(column)
         style['reorderable'] = False
         table = DataTable(source=source, columns=columns, height=self.height,
                           width=self.width, **style)
@@ -80,6 +100,9 @@ class TablePlot(BokehPlot, GenericElementPlot):
         self._execute_hooks(element)
         self.drawn = True
 
+        for cb in self.callbacks:
+            cb.initialize()
+
         return table
 
 
